@@ -1,6 +1,6 @@
-/* $Id: terminal-app.c 26424 2007-12-02 14:13:52Z benny $ */
+/* $Id$ */
 /*-
- * Copyright (c) 2004-2007 os-cillation e.K.
+ * Copyright (c) 2004-2008 os-cillation e.K.
  *
  * Written by Benedikt Meurer <benny@xfce.org>.
  *
@@ -54,12 +54,15 @@ static GtkWidget           *terminal_app_create_window          (TerminalApp
                                                                  gboolean            fullscreen,
                                                                  TerminalVisibility  menubar,
                                                                  TerminalVisibility  borders,
-                                                                 TerminalVisibility  toolbars);
+                                                                 TerminalVisibility  toolbars,
+                                                                 gboolean            maximize);
 static void               terminal_app_new_window               (TerminalWindow     *window,
                                                                  const gchar        *working_directory,
                                                                  TerminalApp        *app);
 static void               terminal_app_new_window_with_terminal (TerminalWindow     *window,
                                                                  TerminalScreen     *terminal,
+                                                                 gint                x,
+                                                                 gint                y,
                                                                  TerminalApp        *app);
 static void               terminal_app_window_destroyed         (GtkWidget          *window,
                                                                  TerminalApp        *app);
@@ -191,11 +194,12 @@ terminal_app_create_window (TerminalApp       *app,
                             gboolean           fullscreen,
                             TerminalVisibility menubar,
                             TerminalVisibility borders,
-                            TerminalVisibility toolbars)
+                            TerminalVisibility toolbars,
+                            gboolean           maximize)
 {
   GtkWidget *window;
 
-  window = terminal_window_new (fullscreen, menubar, borders, toolbars);
+  window = terminal_window_new (fullscreen, menubar, borders, toolbars, maximize);
   g_signal_connect (G_OBJECT (window), "destroy",
                     G_CALLBACK (terminal_app_window_destroyed), app);
   g_signal_connect (G_OBJECT (window), "new-window",
@@ -253,15 +257,26 @@ terminal_app_new_window (TerminalWindow *window,
 static void
 terminal_app_new_window_with_terminal (TerminalWindow *existing,
                                        TerminalScreen *terminal,
+                                       gint            x,
+                                       gint            y,
                                        TerminalApp    *app)
 {
   GtkWidget *window;
   GdkScreen *screen;
+  
+  _terminal_return_if_fail (TERMINAL_IS_WINDOW (existing));
+  _terminal_return_if_fail (TERMINAL_IS_SCREEN (terminal));
+  _terminal_return_if_fail (TERMINAL_IS_APP (app));
 
   window = terminal_app_create_window (app, FALSE,
                                        TERMINAL_VISIBILITY_DEFAULT,
                                        TERMINAL_VISIBILITY_DEFAULT,
-                                       TERMINAL_VISIBILITY_DEFAULT);
+                                       TERMINAL_VISIBILITY_DEFAULT,
+                                       FALSE);
+
+  /* set new window position */
+  if (x > -1 && y > -1)
+    gtk_window_move (GTK_WINDOW (window), x, y);
 
   /* place the new window on the same screen as
    * the existing window.
@@ -347,9 +362,7 @@ static GdkScreen*
 terminal_app_find_screen (const gchar *display_name)
 {
   const gchar *other_name;
-#if GTK_CHECK_VERSION(2,10,0)
   GdkColormap *colormap;
-#endif
   GdkDisplay  *display = NULL;
   GdkScreen   *screen = NULL;
   GSList      *displays;
@@ -410,7 +423,6 @@ terminal_app_find_screen (const gchar *display_name)
       g_object_ref (G_OBJECT (screen));
     }
 
-#if GTK_CHECK_VERSION(2,10,0)
   /* check if we already checked this screen */
   if (g_object_get_data (G_OBJECT (screen), "terminal-checked-screen") == NULL)
     {
@@ -427,7 +439,6 @@ terminal_app_find_screen (const gchar *display_name)
       /* mark this screen as handled */
       g_object_set_data (G_OBJECT (screen), I_("terminal-checked-screen"), GINT_TO_POINTER (1));
     }
-#endif
 
   return screen;
 }
@@ -504,7 +515,8 @@ terminal_app_open_window (TerminalApp         *app,
                                        attr->fullscreen,
                                        attr->menubar,
                                        attr->borders,
-                                       attr->toolbars);
+                                       attr->toolbars,
+                                       attr->maximize);
 
   if (attr->role != NULL)
     gtk_window_set_role (GTK_WINDOW (window), attr->role);
