@@ -1,4 +1,4 @@
-/* $Id: terminal-screen.c 26424 2007-12-02 14:13:52Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2004-2007 os-cillation e.K.
  *
@@ -44,6 +44,7 @@
 #include <terminal/terminal-private.h>
 #include <terminal/terminal-screen.h>
 #include <terminal/terminal-widget.h>
+#include <terminal/terminal-window.h>
 
 #if defined(GDK_WINDOWING_X11)
 #include <gdk/gdkx.h>
@@ -56,6 +57,7 @@ enum
   PROP_0,
   PROP_CUSTOM_TITLE,
   PROP_TITLE,
+  PROP_ACTIVITY,
 };
 
 enum
@@ -107,6 +109,8 @@ static void       terminal_screen_vte_selection_changed         (VteTerminal
                                                                  TerminalScreen        *screen);
 static void       terminal_screen_vte_window_title_changed      (VteTerminal           *terminal,
                                                                  TerminalScreen        *screen);
+static void       terminal_screen_vte_window_contents_changed   (VteTerminal           *terminal,
+                                                                 TerminalScreen        *screen);
 static gboolean   terminal_screen_timer_background              (gpointer               user_data);
 static void       terminal_screen_timer_background_destroy      (gpointer               user_data);
 
@@ -138,6 +142,9 @@ struct _TerminalScreen
 
   guint                background_timer_id;
   guint                launch_idle_id;
+  
+  gboolean             activity;
+  guint                reset_activity_cb;
 };
 
 
@@ -187,6 +194,17 @@ terminal_screen_class_init (TerminalScreenClass *klass)
                                                         NULL,
                                                         G_PARAM_READABLE));
 
+  /**
+   * TerminalScreen:activity:
+   **/
+  g_object_class_install_property (gobject_class,
+                                   PROP_ACTIVITY,
+                                   g_param_spec_boolean ("activity",
+                                                         "activity",
+                                                         "activity",
+                                                         FALSE,
+                                                         G_PARAM_READWRITE));
+
   /**
    * TerminalScreen::get-context-menu
    **/
@@ -295,6 +313,12 @@ terminal_screen_init (TerminalScreen *screen)
   terminal_screen_update_scrolling_on_keystroke (screen);
   terminal_screen_update_word_chars (screen);
   terminal_screen_timer_background (TERMINAL_SCREEN (screen));
+
+  /* Last, connect contents-changed to avoid a race with updates above */
+  g_object_connect (G_OBJECT (screen->terminal),
+                    "signal::contents-changed", G_CALLBACK (terminal_screen_vte_window_contents_changed), screen,
+                    NULL);
+
 }
 
 
@@ -340,6 +364,10 @@ terminal_screen_get_property (GObject          *object,
       g_value_take_string (value, terminal_screen_get_title (screen));
       break;
 
+    case PROP_ACTIVITY:
+      g_value_set_boolean (value, screen->activity);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -362,6 +390,10 @@ terminal_screen_set_property (GObject          *object,
       terminal_screen_set_custom_title (screen, g_value_get_string (value));
       break;
 
+    case PROP_ACTIVITY:
+      screen->activity = g_value_get_boolean (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -373,7 +405,7 @@ terminal_screen_set_property (GObject          *object,
 static void
 terminal_screen_realize (GtkWidget *widget)
 {
-#if GTK_CHECK_VERSION(2,10,0) && defined(HAVE_VTE_TERMINAL_SET_OPACITY)
+#ifdef HAVE_VTE_TERMINAL_SET_OPACITY
   GdkScreen *screen;
 #endif
 
@@ -383,7 +415,7 @@ terminal_screen_realize (GtkWidget *widget)
   if (!GTK_WIDGET_REALIZED (TERMINAL_SCREEN (widget)->terminal))
     gtk_widget_realize (TERMINAL_SCREEN (widget)->terminal);
 
-#if GTK_CHECK_VERSION(2,10,0) && defined(HAVE_VTE_TERMINAL_SET_OPACITY)
+#ifdef HAVE_VTE_TERMINAL_SET_OPACITY
   /* connect to the "composited-changed" signal */
   screen = gtk_widget_get_screen (widget);
   g_signal_connect_swapped (G_OBJECT (screen), "composited-changed", G_CALLBACK (terminal_screen_update_background), widget);
@@ -395,7 +427,7 @@ terminal_screen_realize (GtkWidget *widget)
 static void
 terminal_screen_unrealize (GtkWidget *widget)
 {
-#if GTK_CHECK_VERSION(2,10,0) && defined(HAVE_VTE_TERMINAL_SET_OPACITY)
+#ifdef HAVE_VTE_TERMINAL_SET_OPACITY
   GdkScreen *screen;
 
   /* disconnect the "composited-changed" handler */
@@ -597,22 +629,6 @@ terminal_screen_update_binding_delete (TerminalScreen *screen)
     }
 }
 
-
-
-static void
-query_color (TerminalPreferences *preferences,
-             const gchar         *property,
-             GdkColor            *color_return)
-{
-  gchar *spec;
-
-  g_object_get (G_OBJECT (preferences), property, &spec, NULL);
-  gdk_color_parse (spec, color_return);
-  g_free (spec);
-}
-
-
-
 static void
 terminal_screen_update_colors (TerminalScreen *screen)
 {
@@ -689,7 +705,12 @@ terminal_screen_update_misc_cursor_blinks (TerminalScreen *screen)
 {
   gboolean bval;
   g_object_get (G_OBJECT (screen->preferences), "misc-cursor-blinks", &bval, NULL);
+#if VTE_CHECK_VERSION (0, 17, 1)
+  vte_terminal_set_cursor_blink_mode (VTE_TERMINAL (screen->terminal), 
+                                      bval ? VTE_CURSOR_BLINK_ON : VTE_CURSOR_BLINK_OFF);
+#else
   vte_terminal_set_cursor_blinks (VTE_TERMINAL (screen->terminal), bval);
+#endif
 }
 
 
@@ -843,7 +864,44 @@ terminal_screen_vte_window_title_changed (VteTerminal    *terminal,
   g_object_notify (G_OBJECT (screen), "title");
 }
 
+static gboolean terminal_screen_reset_activity(TerminalScreen *screen)
+{
+  screen->activity = FALSE;
+  screen->reset_activity_cb = 0;
+  g_object_notify (G_OBJECT (screen), "activity");
+  return FALSE;
+}
 
+static void
+terminal_screen_vte_window_contents_changed (VteTerminal    *terminal,
+                                             TerminalScreen *screen)
+{
+  gdouble timeout_seconds = 2.0;
+
+  _terminal_return_if_fail (VTE_IS_TERMINAL (terminal));
+  _terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
+
+  if (terminal_window_is_screen_active (screen))
+    return;
+
+  g_object_get (G_OBJECT (screen->preferences), "tab-activity-timeout", &timeout_seconds, NULL);
+  if (timeout_seconds < 1.0)
+    return;
+
+  screen->activity = TRUE;
+  g_object_notify (G_OBJECT (screen), "activity");
+  if (screen->reset_activity_cb != 0) {
+    g_source_remove(screen->reset_activity_cb);
+  }
+
+#if GLIB_CHECK_VERSION (2,14,0)
+  screen->reset_activity_cb = g_timeout_add_seconds ((gint)timeout_seconds,
+                     (GSourceFunc)terminal_screen_reset_activity, screen);
+#else
+  screen->reset_activity_cb = g_timeout_add ((gint)timeout_seconds * 1000,
+                     (GSourceFunc)terminal_screen_reset_activity, screen);
+#endif
+}
 
 static gboolean
 terminal_screen_timer_background (gpointer user_data)
@@ -854,6 +912,11 @@ terminal_screen_timer_background (gpointer user_data)
   GdkPixbuf           *image;
   gdouble              background_darkness;
 
+  _terminal_return_val_if_fail (TERMINAL_IS_SCREEN (screen), FALSE);
+  _terminal_return_val_if_fail (VTE_IS_TERMINAL (screen->terminal), FALSE);
+
+  GDK_THREADS_ENTER ();
+
   g_object_get (G_OBJECT (screen->preferences), "background-mode", &background_mode, NULL);
 
   if (background_mode == TERMINAL_BACKGROUND_SOLID)
@@ -861,7 +924,7 @@ terminal_screen_timer_background (gpointer user_data)
       vte_terminal_set_background_image (VTE_TERMINAL (screen->terminal), NULL);
       vte_terminal_set_background_saturation (VTE_TERMINAL (screen->terminal), 1.0);
       vte_terminal_set_background_transparent (VTE_TERMINAL (screen->terminal), FALSE);
-#if GTK_CHECK_VERSION(2,10,0) && defined(HAVE_VTE_TERMINAL_SET_OPACITY)
+#ifdef HAVE_VTE_TERMINAL_SET_OPACITY
       vte_terminal_set_opacity (VTE_TERMINAL (screen->terminal), 0xFFFF);
 #endif
     }
@@ -875,7 +938,7 @@ terminal_screen_timer_background (gpointer user_data)
                                           screen->terminal->allocation.width,
                                           screen->terminal->allocation.height);
       vte_terminal_set_background_image (VTE_TERMINAL (screen->terminal), image);
-#if GTK_CHECK_VERSION(2,10,0) && defined(HAVE_VTE_TERMINAL_SET_OPACITY)
+#ifdef HAVE_VTE_TERMINAL_SET_OPACITY
       vte_terminal_set_opacity (VTE_TERMINAL (screen->terminal), 0xFFFF);
 #endif
       if (image != NULL)
@@ -887,7 +950,7 @@ terminal_screen_timer_background (gpointer user_data)
       g_object_get (G_OBJECT (screen->preferences), "background-darkness", &background_darkness, NULL);
       vte_terminal_set_background_image (VTE_TERMINAL (screen->terminal), NULL);
 
-#if GTK_CHECK_VERSION(2,10,0) && defined(HAVE_VTE_TERMINAL_SET_OPACITY)
+#ifdef HAVE_VTE_TERMINAL_SET_OPACITY
       /* check if the X screen is composited */
       if (gdk_screen_is_composited (gtk_widget_get_screen (user_data)))
         {
@@ -900,12 +963,14 @@ terminal_screen_timer_background (gpointer user_data)
 #endif
           vte_terminal_set_background_saturation (VTE_TERMINAL (screen->terminal), 1.0 - background_darkness);
           vte_terminal_set_background_transparent (VTE_TERMINAL (screen->terminal), TRUE);
-#if GTK_CHECK_VERSION(2,10,0) && defined(HAVE_VTE_TERMINAL_SET_OPACITY)
+#ifdef HAVE_VTE_TERMINAL_SET_OPACITY
           vte_terminal_set_opacity (VTE_TERMINAL (screen->terminal), 0xFFFF);
         }
 #endif
     }
 
+  GDK_THREADS_LEAVE ();
+
   return FALSE;
 }
 
@@ -1071,12 +1136,11 @@ terminal_screen_force_resize_window (TerminalScreen *screen,
   gint           xpad;
   gint           ypad;
 
-  gtk_widget_set_size_request (screen->terminal, 2000, 2000);
   gtk_widget_size_request (GTK_WIDGET (window), &window_requisition);
   gtk_widget_size_request (screen->terminal, &terminal_requisition);
 
-  width = window_requisition.width - terminal_requisition.width;
-  height = window_requisition.height - terminal_requisition.height;
+  width = MAX (window_requisition.width - terminal_requisition.width, 0);
+  height = MAX (window_requisition.height - terminal_requisition.height, 0);
 
   if (force_columns < 0)
     columns = VTE_TERMINAL (screen->terminal)->column_count;
@@ -1226,8 +1290,6 @@ terminal_screen_get_title (TerminalScreen *screen)
   return title;
 }
 
-
-
 /**
  * terminal_screen_get_working_directory:
  * @screen      : A #TerminalScreen.
@@ -1273,7 +1335,7 @@ terminal_screen_get_working_directory (TerminalScreen *screen)
                 {
                   g_free (screen->working_directory);
                   screen->working_directory = g_get_current_dir ();
-                  chdir (cwd);
+                  if (chdir (cwd) == 0);
                 }
 
               g_free (cwd);
