@@ -1,6 +1,6 @@
-/* $Id: terminal-window.h 26424 2007-12-02 14:13:52Z benny $ */
+/* $Id$ */
 /*-
- * Copyright (c) 2004-2006 os-cillation e.K.
+ * Copyright (c) 2004-2008 os-cillation e.K.
  *
  * Written by Benedikt Meurer <benny@xfce.org>.
  *
@@ -23,6 +23,7 @@
 #define __TERMINAL_WINDOW_H__
 
 #include <terminal/terminal-screen.h>
+#include <terminal/terminal-options.h>
 
 G_BEGIN_DECLS;
 
@@ -44,7 +45,9 @@ struct _TerminalWindowClass
   void  (*new_window)             (TerminalWindow *window,
                                    const gchar    *working_directory);
   void  (*new_window_with_screen) (TerminalWindow *window,
-                                   TerminalScreen *screen);
+                                   TerminalScreen *screen,
+                                   gint            x,
+                                   gint            y);
 };
 
 GType           terminal_window_get_type             (void) G_GNUC_CONST;
@@ -52,7 +55,8 @@ GType           terminal_window_get_type             (void) G_GNUC_CONST;
 GtkWidget      *terminal_window_new                  (gboolean            fullscreen,
                                                       TerminalVisibility  menubar,
                                                       TerminalVisibility  borders,
-                                                      TerminalVisibility  toolbars);
+                                                      TerminalVisibility  toolbars,
+                                                      gboolean            maximize);
 
 void            terminal_window_add                  (TerminalWindow     *window,
                                                       TerminalScreen     *screen);
@@ -67,6 +71,8 @@ void            terminal_window_set_startup_id       (TerminalWindow     *window
 
 GList          *terminal_window_get_restart_command  (TerminalWindow     *window);
 
+gboolean        terminal_window_is_screen_active     (TerminalScreen     *screen);
+
 G_END_DECLS;
 
 #endif /* !__TERMINAL_WINDOW_H__ */
